@@ -22,22 +22,19 @@ Not persistent ! (no data retained after process exit)
 
 from BTrees.OOBTree import OOBTree as _OOBTree
 import neo.lib
-from hashlib import md5
+from hashlib import sha1
 
 from neo.storage.database import DatabaseManager
-from neo.lib.protocol import CellStates, ZERO_OID, ZERO_TID
+from neo.storage.database.manager import CreationUndone
+from neo.lib.protocol import CellStates, ZERO_HASH, ZERO_OID, ZERO_TID
 from neo.lib import util
 
-# The only purpose of this value (and code using it) is to avoid creating
-# arbitrarily-long lists of values when cleaning up dictionaries.
-KEY_BATCH_SIZE = 1000
-
 # Keep dropped trees in memory to avoid instanciating when not needed.
 TREE_POOL = []
 # How many empty BTree istance to keep in ram
 MAX_TREE_POOL_SIZE = 100
 
-def batchDelete(tree, tester_callback, iter_kw=None, recycle_subtrees=False):
+def batchDelete(tree, tester_callback=None, deleter_callback=None, **kw):
     """
     Iter over given BTree and delete found entries.
     tree BTree
@@ -45,49 +42,21 @@ def batchDelete(tree, tester_callback, iter_kw=None, recycle_subtrees=False):
     tester_callback function(key, value) -> boolean
         Called with each key, value pair found in tree.
         If return value is true, delete entry. Otherwise, skip to next key.
-    iter_kw dict
+    deleter_callback function(tree, key_list) -> None (None)
+        Custom function to delete items
+    **kw
         Keyword arguments for tree.items .
-        Warning: altered in this function.
-    recycle_subtrees boolean (False)
-        If true, deleted values will be put in TREE_POOL for future reuse.
-        They must be BTrees.
-        If False, values are not touched.
     """
-    if iter_kw is None:
-        iter_kw = {}
-    if recycle_subtrees:
-        deleter_callback = _btreeDeleterCallback
+    if tester_callback is None:
+        key_list = list(safeIter(tree.iterkeys, **kw))
     else:
-        deleter_callback = _deleterCallback
-    items = tree.items
-    while True:
-        to_delete = []
-        append = to_delete.append
-        for key, value in safeIter(items, **iter_kw):
-            if tester_callback(key, value):
-                append(key)
-                if len(to_delete) >= KEY_BATCH_SIZE:
-                    iter_kw['min'] = key
-                    iter_kw['excludemin'] = True
-                    break
-        if to_delete:
-            deleter_callback(tree, to_delete)
-        else:
-            break
-
-def _deleterCallback(tree, key_list):
-    for key in key_list:
-        del tree[key]
-
-if hasattr(_OOBTree, 'pop'):
-    def _btreeDeleterCallback(tree, key_list):
+        key_list = [key for key, value in safeIter(tree.iteritems, **kw)
+                        if tester_callback(key, value)]
+    if deleter_callback is None:
         for key in key_list:
-            prune(tree.pop(key))
-else:
-    def _btreeDeleterCallback(tree, key_list):
-        for key in key_list:
-            prune(tree[key])
             del tree[key]
+    else:
+        deleter_callback(tree, key_list)
 
 def OOBTree():
     try:
@@ -112,9 +81,6 @@ def _noPrune(_):
 
 prune = _prune
 
-class CreationUndone(Exception):
-    pass
-
 def iterObjSerials(obj):
     for tserial in obj.values():
         for serial in tserial.keys():
@@ -155,24 +121,20 @@ def safeIter(func, *args, **kw):
 
 class BTreeDatabaseManager(DatabaseManager):
 
-    _obj = None
-    _trans = None
-    _tobj = None
-    _ttrans = None
-    _pt = None
-    _config = None
-
     def __init__(self, database):
         super(BTreeDatabaseManager, self).__init__()
         self.setup(reset=1)
 
     def setup(self, reset=0):
         if reset:
+            self._data = OOBTree()
             self._obj = OOBTree()
             self._trans = OOBTree()
-            self.dropUnfinishedData()
+            self._tobj = OOBTree()
+            self._ttrans = OOBTree()
             self._pt = {}
             self._config = {}
+            self._uncommitted_data = {}
 
     def _begin(self):
         pass
@@ -251,29 +213,6 @@ class BTreeDatabaseManager(DatabaseManager):
                 result = False
         return result
 
-    def _getObjectData(self, oid, value_serial, tid):
-        if value_serial is None:
-            raise CreationUndone
-        if value_serial >= tid:
-            raise ValueError, "Incorrect value reference found for " \
-                "oid %d at tid %d: reference = %d" % (oid, value_serial, tid)
-        try:
-            tserial = self._obj[oid]
-        except KeyError:
-            raise IndexError(oid)
-        try:
-            compression, checksum, value, next_value_serial = tserial[
-                value_serial]
-        except KeyError:
-            raise IndexError(value_serial)
-        if value is None:
-            neo.lib.logging.info("Multiple levels of indirection when " \
-                "searching for object data for oid %d at tid %d. This " \
-                "causes suboptimal performance." % (oid, value_serial))
-            value_serial, compression, checksum, value = self._getObjectData(
-                oid, next_value_serial, value_serial)
-        return value_serial, compression, checksum, value
-
     def _getObject(self, oid, tid=None, before_tid=None):
         tserial = self._obj.get(oid)
         if tserial is not None:
@@ -284,14 +223,20 @@ class BTreeDatabaseManager(DatabaseManager):
                     else:
                         tid = tserial.maxKey(before_tid - 1)
                 except ValueError:
-                    return
-            result = tserial.get(tid)
-            if result:
-                try:
-                    next_serial = tserial.minKey(tid + 1)
-                except ValueError:
-                    next_serial = None
-                return (tid, next_serial) + result
+                    return False
+            try:
+                checksum, value_serial = tserial[tid]
+            except KeyError:
+                return False
+            try:
+                next_serial = tserial.minKey(tid + 1)
+            except ValueError:
+                next_serial = None
+            if checksum is None:
+                compression = data = None
+            else:
+                compression, data, _ = self._data[checksum]
+            return tid, next_serial, compression, checksum, data, value_serial
 
     def doSetPartitionTable(self, ptid, cell_list, reset):
         pt = self._pt
@@ -313,16 +258,48 @@ class BTreeDatabaseManager(DatabaseManager):
     def setPartitionTable(self, ptid, cell_list):
         self.doSetPartitionTable(ptid, cell_list, True)
 
+    def _oidDeleterCallback(self, oid):
+        data = self._data
+        uncommitted_data = self._uncommitted_data
+        def deleter_callback(tree, key_list):
+            for tid in key_list:
+                checksum = tree[tid][0] # BBB: recent ZODB provides pop()
+                del tree[tid]           #
+                if checksum:
+                    index = data[checksum][2]
+                    index.remove((oid, tid))
+                    if not index and checksum not in uncommitted_data:
+                        del data[checksum]
+        return deleter_callback
+
+    def _objDeleterCallback(self, tree, key_list):
+        data = self._data
+        checksum_list = []
+        checksum_set = set()
+        for oid in key_list:
+            tserial = tree[oid]; del tree[oid] # BBB: recent ZODB provides pop()
+            for tid, (checksum, _) in tserial.items():
+                if checksum:
+                    index = data[checksum][2]
+                    try:
+                        index.remove((oid, tid))
+                    except KeyError: # _tobj
+                        checksum_list.append(checksum)
+                    checksum_set.add(checksum)
+            prune(tserial)
+        self.unlockData(checksum_list)
+        self._pruneData(checksum_set)
+
     def dropPartitions(self, num_partitions, offset_list):
         offset_list = frozenset(offset_list)
         def same_partition(key, _):
             return key % num_partitions in offset_list
-        batchDelete(self._obj, same_partition, recycle_subtrees=True)
+        batchDelete(self._obj, same_partition, self._objDeleterCallback)
         batchDelete(self._trans, same_partition)
 
     def dropUnfinishedData(self):
-        self._tobj = OOBTree()
-        self._ttrans = OOBTree()
+        batchDelete(self._tobj, deleter_callback=self._objDeleterCallback)
+        self._ttrans.clear()
 
     def storeTransaction(self, tid, object_list, transaction, temporary=True):
         u64 = util.u64
@@ -333,45 +310,39 @@ class BTreeDatabaseManager(DatabaseManager):
         else:
             obj = self._obj
             trans = self._trans
-        for oid, compression, checksum, data, value_serial in object_list:
+        data = self._data
+        for oid, checksum, value_serial in object_list:
             oid = u64(oid)
-            if data is None:
-                compression = checksum = data
-            else:
-                # TODO: unit-test this raise
-                if value_serial is not None:
-                    raise ValueError, 'Either data or value_serial ' \
-                        'must be None (oid %d, tid %d)' % (oid, tid)
+            if value_serial:
+                value_serial = u64(value_serial)
+                checksum = self._obj[oid][value_serial][0]
+                if temporary:
+                    self.storeData(checksum)
+            if checksum:
+                if not temporary:
+                    data[checksum][2].add((oid, tid))
             try:
                 tserial = obj[oid]
             except KeyError:
                 tserial = obj[oid] = OOBTree()
-            if value_serial is not None:
-                value_serial = u64(value_serial)
-            tserial[tid] = (compression, checksum, data, value_serial)
+            tserial[tid] = checksum, value_serial
 
         if transaction is not None:
             oid_list, user, desc, ext, packed = transaction
             trans[tid] = (tuple(oid_list), user, desc, ext, packed)
 
-    def _getDataTIDFromData(self, oid, result):
-        tid, _, _, _, data, value_serial = result
-        if data is None:
-            try:
-                data_serial = self._getObjectData(oid, value_serial, tid)[0]
-            except CreationUndone:
-                data_serial = None
-        else:
-            data_serial = tid
-        return tid, data_serial
+    def _pruneData(self, checksum_list):
+        data = self._data
+        for checksum in set(checksum_list).difference(self._uncommitted_data):
+            if not data[checksum][2]:
+                del data[checksum]
 
-    def _getDataTID(self, oid, tid=None, before_tid=None):
-        result = self._getObject(oid, tid=tid, before_tid=before_tid)
-        if result is None:
-            result = (None, None)
-        else:
-            result = self._getDataTIDFromData(oid, result)
-        return result
+    def _storeData(self, checksum, data, compression):
+        try:
+            if self._data[checksum][:2] != (compression, data):
+                raise AssertionError("hash collision")
+        except KeyError:
+            self._data[checksum] = compression, data, set()
 
     def finishTransaction(self, tid):
         tid = util.u64(tid)
@@ -386,8 +357,9 @@ class BTreeDatabaseManager(DatabaseManager):
             self._trans[tid] = data
 
     def _popTransactionFromTObj(self, tid, to_obj):
+        checksum_list = []
         if to_obj:
-            recycle_subtrees = False
+            deleter_callback = None
             obj = self._obj
             def callback(oid, data):
                 try:
@@ -395,8 +367,12 @@ class BTreeDatabaseManager(DatabaseManager):
                 except KeyError:
                     tserial = obj[oid] = OOBTree()
                 tserial[tid] = data
+                checksum = data[0]
+                if checksum:
+                    self._data[checksum][2].add((oid, tid))
+                    checksum_list.append(checksum)
         else:
-            recycle_subtrees = True
+            deleter_callback = self._objDeleterCallback
             callback = lambda oid, data: None
         def tester_callback(oid, tserial):
             try:
@@ -407,8 +383,8 @@ class BTreeDatabaseManager(DatabaseManager):
                 del tserial[tid]
                 callback(oid, data)
             return not tserial
-        batchDelete(self._tobj, tester_callback,
-            recycle_subtrees=recycle_subtrees)
+        batchDelete(self._tobj, tester_callback, deleter_callback)
+        self.unlockData(checksum_list)
 
     def deleteTransaction(self, tid, oid_list=()):
         u64 = util.u64
@@ -429,7 +405,7 @@ class BTreeDatabaseManager(DatabaseManager):
         def same_partition(key, _):
             return key % num_partitions == partition
         batchDelete(self._trans, same_partition,
-            iter_kw={'min': util.u64(tid), 'max': util.u64(max_tid)})
+            min=util.u64(tid), max=util.u64(max_tid))
 
     def deleteObject(self, oid, serial=None):
         u64 = util.u64
@@ -440,16 +416,11 @@ class BTreeDatabaseManager(DatabaseManager):
         try:
             tserial = obj[oid]
         except KeyError:
-            pass
-        else:
-            if serial is not None:
-                try:
-                    del tserial[serial]
-                except KeyError:
-                    pass
-            if serial is None or not tserial:
-                prune(obj[oid])
-                del obj[oid]
+            return
+        batchDelete(tserial, deleter_callback=self._oidDeleterCallback(oid),
+                    min=serial, max=serial)
+        if not tserial:
+            del obj[oid]
 
     def deleteObjectsAbove(self, num_partitions, partition, oid, serial,
                            max_tid):
@@ -464,13 +435,14 @@ class BTreeDatabaseManager(DatabaseManager):
             except KeyError:
                 pass
             else:
-                batchDelete(tserial, lambda _, __: True,
-                    iter_kw={'min': serial, 'max': max_tid})
+                batchDelete(tserial, min=serial, max=max_tid,
+                    deleter_callback=self._oidDeleterCallback(oid))
+                if not tserial:
+                    del tserial[oid]
         def same_partition(key, _):
             return key % num_partitions == partition
-        batchDelete(obj, same_partition,
-            iter_kw={'min': oid, 'excludemin': True, 'max': max_tid},
-            recycle_subtrees=True)
+        batchDelete(obj, same_partition, self._objDeleterCallback,
+            min=oid, excludemin=True, max=max_tid)
 
     def getTransaction(self, tid, all=False):
         tid = util.u64(tid)
@@ -506,15 +478,13 @@ class BTreeDatabaseManager(DatabaseManager):
     def _getObjectLength(self, oid, value_serial):
         if value_serial is None:
             raise CreationUndone
-        _, _, value, value_serial = self._obj[oid][value_serial]
-        if value is None:
+        checksum, value_serial = self._obj[oid][value_serial]
+        if checksum is None:
             neo.lib.logging.info("Multiple levels of indirection when " \
                 "searching for object data for oid %d at tid %d. This " \
                 "causes suboptimal performance." % (oid, value_serial))
-            length = self._getObjectLength(oid, value_serial)
-        else:
-            length = len(value)
-        return length
+            return self._getObjectLength(oid, value_serial)
+        return len(self._data[checksum][1])
 
     def getObjectHistory(self, oid, offset=0, length=1):
         # FIXME: This method doesn't take client's current ransaction id as
@@ -534,17 +504,18 @@ class BTreeDatabaseManager(DatabaseManager):
             while offset > 0:
                 tserial_iter.next()
                 offset -= 1
-            for serial, (_, _, value, value_serial) in tserial_iter:
+            data = self._data
+            for serial, (checksum, value_serial) in tserial_iter:
                 if length == 0 or serial < pack_tid:
                     break
                 length -= 1
-                if value is None:
+                if checksum is None:
                     try:
                         data_length = self._getObjectLength(oid, value_serial)
                     except CreationUndone:
                         data_length = 0
                 else:
-                    data_length = len(value)
+                    data_length = len(data[checksum][1])
                 append((p64(serial), data_length))
         if not result:
             result = None
@@ -615,39 +586,28 @@ class BTreeDatabaseManager(DatabaseManager):
                 append(p64(tid))
         return result
 
-    def _updatePackFuture(self, oid, orig_serial, max_serial,
-            updateObjectDataForPack):
-        p64 = util.p64
+    def _updatePackFuture(self, oid, orig_serial, max_serial):
         # Before deleting this objects revision, see if there is any
         # transaction referencing its value at max_serial or above.
         # If there is, copy value to the first future transaction. Any further
         # reference is just updated to point to the new data location.
-        value_serial = None
+        new_serial = None
         obj = self._obj
         for tree in (obj, self._tobj):
             try:
                 tserial = tree[oid]
             except KeyError:
                 continue
-            for serial, record in tserial.items(
+            for serial, (checksum, value_serial) in tserial.iteritems(
                     min=max_serial):
-                if record[3] == orig_serial:
-                    if value_serial is None:
-                        value_serial = serial
-                        tserial[serial] = tserial[orig_serial]
-                    else:
-                        record = list(record)
-                        record[3] = value_serial
-                        tserial[serial] = tuple(record)
-        def getObjectData():
-            assert value_serial is None
-            return obj[oid][orig_serial][:3]
-        if value_serial:
-            value_serial = p64(value_serial)
-        updateObjectDataForPack(p64(oid), p64(orig_serial), value_serial,
-            getObjectData)
+                if value_serial == orig_serial:
+                    tserial[serial] = checksum, new_serial
+                    if not new_serial:
+                        new_serial = serial
+        return new_serial
 
     def pack(self, tid, updateObjectDataForPack):
+        p64 = util.p64
         tid = util.u64(tid)
         updatePackFuture = self._updatePackFuture
         self._setPackTID(tid)
@@ -658,17 +618,21 @@ class BTreeDatabaseManager(DatabaseManager):
                 # No entry before pack TID, nothing to pack on this object.
                 pass
             else:
-                if tserial[max_serial][2] == '':
+                if tserial[max_serial][0] is None:
                     # Last version before/at pack TID is a creation undo, drop
                     # it too.
                     max_serial += 1
-                def serial_callback(serial, _):
-                    updatePackFuture(oid, serial, max_serial,
-                        updateObjectDataForPack)
+                def serial_callback(serial, value):
+                    new_serial = updatePackFuture(oid, serial, max_serial)
+                    if new_serial:
+                        new_serial = p64(new_serial)
+                    updateObjectDataForPack(p64(oid), p64(serial),
+                                            new_serial, value[0])
                 batchDelete(tserial, serial_callback,
-                    iter_kw={'max': max_serial, 'excludemax': True})
+                    self._oidDeleterCallback(oid),
+                    max=max_serial, excludemax=True)
             return not tserial
-        batchDelete(self._obj, obj_callback, recycle_subtrees=True)
+        batchDelete(self._obj, obj_callback, self._objDeleterCallback)
 
     def checkTIDRange(self, min_tid, max_tid, length, num_partitions, partition):
         if length:
@@ -681,9 +645,9 @@ class BTreeDatabaseManager(DatabaseManager):
                         break
             if tid_list:
                 return (len(tid_list),
-                        md5(','.join(map(str, tid_list))).digest(),
+                        sha1(','.join(map(str, tid_list))).digest(),
                         util.p64(tid_list[-1]))
-        return 0, None, ZERO_TID
+        return 0, ZERO_HASH, ZERO_TID
 
     def checkSerialRange(self, min_oid, min_serial, max_tid, length,
             num_partitions, partition):
@@ -714,8 +678,8 @@ class BTreeDatabaseManager(DatabaseManager):
             if oid_list:
                 p64 = util.p64
                 return (len(oid_list),
-                        md5(','.join(map(str, oid_list))).digest(),
+                        sha1(','.join(map(str, oid_list))).digest(),
                         p64(oid_list[-1]),
-                        md5(','.join(map(str, serial_list))).digest(),
+                        sha1(','.join(map(str, serial_list))).digest(),
                         p64(serial_list[-1]))
-        return 0, None, ZERO_OID, None, ZERO_TID
+        return 0, ZERO_HASH, ZERO_OID, ZERO_HASH, ZERO_TID
